@@ -29,11 +29,6 @@
 
 #include "kleopatra_debug.h"
 
-#include <gpgme++/gpgmepp_version.h>
-#if GPGMEPP_VERSION >= 0x10E01 // 1.14.1
-# define GPGME_HAS_REVSIG
-#endif
-
 using namespace Kleo;
 using namespace Kleo::Commands;
 using namespace GpgME;
@@ -59,7 +54,7 @@ private:
 
 private:
     void ensureDialogCreated();
-    void createJob();
+    QGpgME::QuickJob *createJob();
 
 private:
     Key certificationKey;
@@ -106,11 +101,21 @@ void RevokeCertificationCommand::Private::init()
 
 void RevokeCertificationCommand::Private::slotDialogAccepted()
 {
-    createJob();
+    const auto certificationKey = dialog->selectedCertificationKey();
+    const auto selectedUserIDs = dialog->selectedUserIDs();
+    if (certificationKey.isNull() || selectedUserIDs.empty()) {
+        qCDebug(KLEOPATRA_LOG) << "No certification key or no user IDs selected -> skipping revocation";
+        finished();
+        return;
+    }
 
-#ifdef GPGME_HAS_REVSIG
+    job = createJob();
+    if (!job) {
+        qCDebug(KLEOPATRA_LOG) << "Failed to create QuickJob";
+        finished();
+        return;
+    }
     job->startRevokeSignature(certificationTarget, dialog->selectedCertificationKey(), dialog->selectedUserIDs());
-#endif
 }
 
 void RevokeCertificationCommand::Private::slotDialogRejected()
@@ -154,27 +159,25 @@ void RevokeCertificationCommand::Private::ensureDialogCreated()
     connect(dialog, SIGNAL(rejected()), q, SLOT(slotDialogRejected()));
 }
 
-void RevokeCertificationCommand::Private::createJob()
+QGpgME::QuickJob *RevokeCertificationCommand::Private::createJob()
 {
     Q_ASSERT(!job);
 
     Q_ASSERT(certificationTarget.protocol() == OpenPGP);
     const auto backend = QGpgME::openpgp();
     if (!backend) {
-        return;
+        return nullptr;
     }
 
     QuickJob *const j = backend->quickJob();
-    if (!j) {
-        return;
+    if (j) {
+        connect(j, &Job::progress,
+                q, &Command::progress);
+        connect(j, SIGNAL(result(GpgME::Error)),
+                q, SLOT(slotResult(GpgME::Error)));
     }
 
-    connect(j, &Job::progress,
-            q, &Command::progress);
-    connect(j, SIGNAL(result(GpgME::Error)),
-            q, SLOT(slotResult(GpgME::Error)));
-
-    job = j;
+    return j;
 }
 
 RevokeCertificationCommand::RevokeCertificationCommand(QAbstractItemView *v, KeyListController *c)
@@ -206,11 +209,7 @@ RevokeCertificationCommand::~RevokeCertificationCommand()
 // static
 bool RevokeCertificationCommand::isSupported()
 {
-#ifdef GPGME_HAS_REVSIG
     return engineInfo(GpgEngine).engineVersion() >= "2.2.24";
-#else
-    return false;
-#endif
 }
 
 void RevokeCertificationCommand::doStart()
