@@ -121,11 +121,13 @@ SignClipboardCommand::~SignClipboardCommand()
 // static
 bool SignClipboardCommand::canSignCurrentClipboard()
 {
-    if (const QClipboard *clip = QApplication::clipboard())
-        if (const QMimeData *mime = clip->mimeData()) {
-            return mime->hasText();
+    bool canSign = false;
+    if (const QClipboard *const clip = QApplication::clipboard()) {
+        if (const QMimeData *const mime = clip->mimeData()) {
+            canSign = mime->hasText();
         }
-    return false;
+    }
+    return canSign;
 }
 
 void SignClipboardCommand::doStart()
