@@ -7,8 +7,7 @@
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
-#ifndef __KLEOPATRA_COMMMANDS_SIGNCLIPBOARDCOMMAND_H__
-#define __KLEOPATRA_COMMMANDS_SIGNCLIPBOARDCOMMAND_H__
+#pragma once
 
 #include <commands/command.h>
 
@@ -51,4 +50,3 @@ private:
 
 #endif // QT_NO_CLIPBOARD
 
-#endif // __KLEOPATRA_COMMMANDS_SIGNCLIPBOARDCOMMAND_H__
