@@ -3,30 +3,27 @@
     This file is part of Kleopatra, the KDE keymanager
     SPDX-FileCopyrightText: 2016 Bundesamt für Sicherheit in der Informationstechnik
     SPDX-FileContributor: Intevation GmbH
-    SPDX-FileCopyrightText: 2021 g10 Code GmbH
+    SPDX-FileCopyrightText: 2021, 2022 g10 Code GmbH
     SPDX-FileContributor: Ingo Klöcker <dev@ingo-kloecker.de>
 
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 #pragma once
 
-#include <Libkleo/KeyGroup>
-
-#include <QLineEdit>
-
-#include <gpgme++/key.h>
+#include <QWidget>
 
 #include <memory>
 
-class QCompleter;
-class QLabel;
-class QAction;
+namespace GpgME
+{
+class Key;
+}
 
 namespace Kleo
 {
 class AbstractKeyListModel;
 class KeyFilter;
-class KeyListSortFilterProxyModel;
+class KeyGroup;
 
 /** Line edit and completion based Certificate Selection Widget.
  *
@@ -35,7 +32,7 @@ class KeyListSortFilterProxyModel;
  * The widget will use a single line HBox Layout. For larger dialog
  * see certificateslectiondialog.
  */
-class CertificateLineEdit: public QLineEdit
+class CertificateLineEdit: public QWidget
 {
     Q_OBJECT
 public:
@@ -48,15 +45,20 @@ public:
      * @param parent: The usual widget parent.
      * @param filter: The filters to use. See certificateselectiondialog.
      */
-    CertificateLineEdit(AbstractKeyListModel *model,
-                        QWidget *parent = nullptr,
-                        KeyFilter *filter = nullptr);
+    explicit CertificateLineEdit(AbstractKeyListModel *model,
+                                 KeyFilter *filter = nullptr,
+                                 QWidget *parent = nullptr);
+
+    ~CertificateLineEdit() override;
 
     /** Get the selected key */
     GpgME::Key key() const;
 
     KeyGroup group() const;
 
+    /** The current text */
+    QString text() const;
+
     /** Check if the text is empty */
     bool isEmpty() const;
 
@@ -69,6 +71,8 @@ public:
     /** Set the used keyfilter. */
     void setKeyFilter(const std::shared_ptr<KeyFilter> &filter);
 
+    void setAccessibleNameOfLineEdit(const QString &name);
+
 Q_SIGNALS:
     /** Emitted when the selected key changed. */
     void keyChanged();
@@ -79,27 +83,12 @@ Q_SIGNALS:
     /** Emitted when the entry is no longer empty. */
     void editingStarted();
 
-    /** Emitted when the details dialog or the selection dialog is requested. */
-    void dialogRequested();
-
-private Q_SLOTS:
-    void updateKey();
-    void editChanged();
-    void editFinished();
-    void checkLocate();
+    /** Emitted when the certificate selection dialog is requested. */
+    void certificateSelectionRequested();
 
 private:
-    KeyListSortFilterProxyModel *const mFilterModel;
-    KeyListSortFilterProxyModel *const mCompleterFilterModel;
-    QCompleter *mCompleter = nullptr;
-    QLabel *mStatusLabel,
-           *mStatusIcon;
-    GpgME::Key mKey;
-    KeyGroup mGroup;
-    GpgME::Protocol mCurrentProto;
-    std::shared_ptr<KeyFilter> mFilter;
-    bool mEditStarted = false;
-    bool mEditFinished = false;
-    QAction *const mLineAction;
+    class Private;
+    std::unique_ptr<Private> const d;
 };
+
 }
