@@ -25,7 +25,6 @@
 #include "commands/importcertificatefromfilecommand.h"
 #include "commands/decryptverifyfilescommand.h"
 #include "commands/signencryptfilescommand.h"
-#include "commands/listreaderscommand.h"
 
 #include "conf/groupsconfigdialog.h"
 
@@ -70,11 +69,13 @@
 #include <QLabel>
 
 #include <Libkleo/Formatting>
+#include <Libkleo/GnuPG>
 #include <Libkleo/KeyListModel>
 #include <Libkleo/KeyListSortFilterProxyModel>
 #include <Libkleo/Stl_Util>
 #include <Libkleo/Classify>
 #include <Libkleo/KeyCache>
+#include <Libkleo/DocAction>
 
 #include <vector>
 #include <KSharedConfig>
@@ -87,16 +88,16 @@ using namespace GpgME;
 
 static KGuiItem KStandardGuiItem_quit()
 {
-    static const QString app = KAboutData::applicationData().componentName();
+    static const QString app = KAboutData::applicationData().displayName();
     KGuiItem item = KStandardGuiItem::quit();
-    item.setText(i18nc("Quit [ApplicationName]", "&Quit %1", app));
+    item.setText(xi18nc("@action:button", "&Quit <application>%1</application>", app));
     return item;
 }
 
 static KGuiItem KStandardGuiItem_close()
 {
     KGuiItem item = KStandardGuiItem::close();
-    item.setText(i18n("Only &Close Window"));
+    item.setText(i18nc("@action:button", "Only &Close Window"));
     return item;
 }
 
@@ -143,11 +144,14 @@ public:
 
     void closeAndQuit()
     {
-        const QString app = KAboutData::applicationData().componentName();
+        const QString app = KAboutData::applicationData().displayName();
         const int rc = KMessageBox::questionYesNoCancel(q,
-                       i18n("%1 may be used by other applications as a service.\n"
-                            "You may instead want to close this window without exiting %1.", app),
-                       i18n("Really Quit?"), KStandardGuiItem_close(), KStandardGuiItem_quit(), KStandardGuiItem::cancel(),
+                       xi18n("<application>%1</application> may be used by other applications as a service.<nl/>"
+                             "You may instead want to close this window without exiting <application>%1</application>.", app),
+                       i18nc("@title:window", "Really Quit?"),
+                       KStandardGuiItem_close(),
+                       KStandardGuiItem_quit(),
+                       KStandardGuiItem::cancel(),
                        QLatin1String("really-quit-") + app.toLower());
         if (rc == KMessageBox::Cancel) {
             return;
@@ -184,7 +188,7 @@ public:
             auto statusBar = std::make_unique<QStatusBar>();
             auto statusLbl = std::make_unique<QLabel>(Formatting::deVsString(Kleo::gnupgIsDeVsCompliant()));
             const auto color = KColorScheme(QPalette::Active, KColorScheme::View).foreground(
-                Kleo::gnupgIsDeVsCompliant() ? KColorScheme::PositiveText : KColorScheme::NegativeText
+                Kleo::gnupgIsDeVsCompliant() ? KColorScheme::NormalText: KColorScheme::NegativeText
             ).color();
             const auto background = KColorScheme(QPalette::Active, KColorScheme::View).background(
                 Kleo::gnupgIsDeVsCompliant() ? KColorScheme::PositiveBackground : KColorScheme::NegativeBackground
@@ -216,10 +220,17 @@ public:
 
     void gnupgLogViewer()
     {
-        if (!QProcess::startDetached(QStringLiteral("kwatchgnupg"), QStringList()))
-            KMessageBox::error(q, i18n("Could not start the GnuPG Log Viewer (kwatchgnupg). "
-                                       "Please check your installation."),
-                               i18n("Error Starting KWatchGnuPG"));
+      const QString exec =
+          QStandardPaths::findExecutable(QStringLiteral("kwatchgnupg"));
+      if (exec.isEmpty()) {
+        KMessageBox::error(
+            q,
+            i18n("Could not start the GnuPG Log Viewer (kwatchgnupg). "
+                 "Please check your installation."),
+            i18n("Error Starting KWatchGnuPG"));
+      } else {
+        QProcess::startDetached(QStringLiteral("kwatchgnupg"), QStringList());
+      }
     }
 
     void forceUpdateCheck()
@@ -227,19 +238,6 @@ public:
         UpdateNotification::forceUpdateCheck(q);
     }
 
-    void openCompendium()
-    {
-        QDir datadir(QCoreApplication::applicationDirPath() + QStringLiteral("/../share/gpg4win"));
-        const auto path = datadir.filePath(i18nc("The Gpg4win compendium is only available"
-                                                 "at this point (24.7.2017) in german and english."
-                                                 "Please check with Gpg4win before translating this filename.",
-                                                 "gpg4win-compendium-en.pdf"));
-        qCDebug(KLEOPATRA_LOG) << "Opening Compendium at:" << path;
-        // The compendium is always installed. So this should work. Otherwise
-        // we have debug output.
-        QDesktopServices::openUrl(QUrl::fromLocalFile(path));
-    }
-
     void slotConfigCommitted();
     void slotContextMenuRequested(QAbstractItemView *, const QPoint &p)
     {
@@ -289,10 +287,9 @@ public:
         ui.stackWidget->resize(ui.padWidget->sizeHint());
     }
 
-    void listSmartcardReaders()
+    void restartDaemons()
     {
-        auto command = new ListReadersCommand(q);
-        command->start();
+        Kleo::killDaemons();
     }
 
 private:
@@ -402,6 +399,10 @@ MainWindow::Private::Private(MainWindow *qq)
 
     updateSearchBarClickMessage();
     updateStatusBar();
+
+    if (KeyCache::instance()->initialized()) {
+        keyListingDone();
+    }
 }
 
 MainWindow::Private::~Private() {}
@@ -417,8 +418,8 @@ void MainWindow::Private::setupActions()
 
     KActionCollection *const coll = q->actionCollection();
 
-    const action_data action_data[] = {
-        // most have been MOVED TO keylistcontroller.cpp
+    const std::vector<action_data> action_data = {
+        // see keylistcontroller.cpp for more actions
         // Tools menu
 #ifndef Q_OS_WIN
         {
@@ -426,16 +427,19 @@ void MainWindow::Private::setupActions()
             "kwatchgnupg", q, SLOT(gnupgLogViewer()), QString(), false, true
         },
 #endif
+        {
+            "tools_restart_backend", i18nc("@action:inmenu", "Restart Background Processes"),
+            i18nc("@info:tooltip", "Restart the background processes, e.g. after making changes to the configuration."),
+            "view-refresh", q, SLOT(restartDaemons()), {}, false, true
+        },
+        // Help menu
 #ifdef Q_OS_WIN
         {
             "help_check_updates", i18n("Check for updates"), QString(),
             "gpg4win-compact", q, SLOT(forceUpdateCheck()), QString(), false, true
         },
-        {
-            "help_show_compendium", i18n("Gpg4win Compendium"), QString(),
-            "gpg4win-compact", q, SLOT(openCompendium()), QString(), false, true
-        },
 #endif
+        // View menu
         {
             "view_certificate_overview", i18nc("@action show certificate overview", "Certificates"),
             i18n("Show certificate overview"), "view-certificate", q, SLOT(showCertificateView()), QString(), false, true
@@ -444,29 +448,22 @@ void MainWindow::Private::setupActions()
             "pad_view", i18nc("@action show input / output area for encrypting/signing resp. decrypting/verifying text", "Notepad"),
             i18n("Show pad for encrypting/decrypting and signing/verifying text"), "note", q, SLOT(showPadView()), QString(), false, true
         },
-        // most have been MOVED TO keylistcontroller.cpp
+        {
+            "manage_smartcard", i18nc("@action show smartcard management view", "Smartcards"),
+            i18n("Show smartcard management"), "auth-sim-locked", q, SLOT(showSmartcardView()), QString(), false, true
+        },
         // Settings menu
         {
             "settings_self_test", i18n("Perform Self-Test"), QString(),
             nullptr, q, SLOT(selfTest()), QString(), false, true
         },
-        {
-            "settings_list_readers", i18n("List Smartcard Readers"), QString(),
-            nullptr, q, SLOT(listSmartcardReaders()), QString(), false, true
-        },
         {
             "configure_groups", i18n("Configure Groups..."), QString(),
             "group", q, SLOT(configureGroups()), QString(), false, true
-        },
-        {
-            "manage_smartcard", i18nc("@action show smartcard management view", "Smartcards"),
-            i18n("Show smartcard management"), "auth-sim-locked", q, SLOT(showSmartcardView()), QString(), false, true
         }
-
-        // most have been MOVED TO keylistcontroller.cpp
     };
 
-    make_actions_from_data(action_data, /*sizeof action_data / sizeof *action_data,*/ coll);
+    make_actions_from_data(action_data, coll);
 
     if (!Settings().groupsEnabled()) {
         if (auto action = coll->action(QStringLiteral("configure_groups"))) {
@@ -480,10 +477,6 @@ void MainWindow::Private::setupActions()
         }
     }
 
-    if (QAction *action = coll->action(QStringLiteral("configure_backend"))) {
-        action->setMenuRole(QAction::NoRole);    //prevent Qt OS X heuristics for config* actions
-    }
-
     KStandardAction::close(q, SLOT(close()), coll);
     KStandardAction::quit(q, SLOT(closeAndQuit()), coll);
     KStandardAction::configureToolbars(q, SLOT(configureToolbars()), coll);
@@ -500,6 +493,46 @@ void MainWindow::Private::setupActions()
     clipboadMenu->clipboardMenu()->setPopupMode(QToolButton::InstantPopup);
     coll->addAction(QStringLiteral("clipboard_menu"), clipboadMenu->clipboardMenu());
 
+    /* Add additional help actions for documentation */
+    const auto compendium = new DocAction(QIcon::fromTheme(QStringLiteral("gpg4win-compact")), i18n("Gpg4win Compendium"),
+            i18nc("The Gpg4win compendium is only available"
+                  "at this point (24.7.2017) in german and english."
+                  "Please check with Gpg4win before translating this filename.",
+                  "gpg4win-compendium-en.pdf"),
+            QStringLiteral("../share/gpg4win"));
+    coll->addAction(QStringLiteral("help_doc_compendium"), compendium);
+
+    /* Documentation centered around the german approved VS-NfD mode for official
+     * RESTRICTED communication. This is only available in some distributions with
+     * the focus on official communications. */
+    const auto symguide = new DocAction(QIcon::fromTheme(QStringLiteral("help-contextual")), i18n("Password-based encryption"),
+            i18nc("Only available in German and English. Leave to English for other languages.",
+                "handout_symmetric_encryption_gnupg_en.pdf"),
+            QStringLiteral("../share/doc/gnupg-vsd"));
+    coll->addAction(QStringLiteral("help_doc_symenc"), symguide);
+
+    const auto quickguide = new DocAction(QIcon::fromTheme(QStringLiteral("help-contextual")), i18n("Quickguide"),
+            i18nc("Only available in German and English. Leave to English for other languages.",
+                "handout_sign_encrypt_gnupg_en.pdf"),
+            QStringLiteral("../share/doc/gnupg-vsd"));
+    coll->addAction(QStringLiteral("help_doc_quickguide"), quickguide);
+
+    const auto vsa10573 = new DocAction(QIcon::fromTheme(QStringLiteral("dvipdf")), i18n("SecOps VSA-10573"),
+            i18nc("Only available in German and English. Leave to English for other languages.",
+                "BSI-VSA-10573-ENG_secops-20220207.pdf"),
+            QStringLiteral("../share/doc/gnupg-vsd"));
+    coll->addAction(QStringLiteral("help_doc_vsa10573"), vsa10573);
+
+    const auto vsa10584 = new DocAction(QIcon::fromTheme(QStringLiteral("dvipdf")), i18n("SecOps VSA-10584"),
+            i18nc("Only available in German and English. Leave to English for other languages.",
+                "BSI-VSA-10584-ENG_secops-20220207.pdf"),
+            QStringLiteral("../share/doc/gnupg-vsd"));
+    coll->addAction(QStringLiteral("help_doc_vsa10584"), vsa10584);
+
+    const auto man_gpg = new DocAction(QIcon::fromTheme(QStringLiteral("help-contextual")), i18n("GnuPG Manual"),
+            QStringLiteral("gnupg.pdf"), QStringLiteral("../share/doc/gnupg"));
+    coll->addAction(QStringLiteral("help_doc_gnupg"), man_gpg);
+
     q->setStandardToolBarMenuEnabled(true);
 
     controller.createActions(coll);
