@@ -11,25 +11,15 @@
 
 #include <config-kleopatra.h>
 
-#include <gpgme++/gpgmepp_version.h>
-#if GPGMEPP_VERSION >= 0x10E01 // 1.14.1
-# define QGPGME_HAS_DEBUG
-# define GPGME_SUPPORTS_API_FOR_DEVICEINFOWATCHER
-#endif
-
 #include "readerstatus.h"
 
-#ifdef GPGME_SUPPORTS_API_FOR_DEVICEINFOWATCHER
-# include "deviceinfowatcher.h"
-#endif
+#include "deviceinfowatcher.h"
 
 #include <Libkleo/Assuan>
 #include <Libkleo/GnuPG>
 #include <Libkleo/FileSystemWatcher>
 
-#ifdef QGPGME_HAS_DEBUG
-# include <QGpgME/Debug>
-#endif
+#include <QGpgME/Debug>
 
 #include <gpgme++/context.h>
 #include <gpgme++/defaultassuantransaction.h>
@@ -93,16 +83,6 @@ static QDebug operator<<(QDebug s, const std::string &string)
     return s << QString::fromStdString(string);
 }
 
-#ifndef QGPGME_HAS_DEBUG
-static QDebug operator<<(QDebug s, const GpgME::Error &err)
-{
-    const bool oldSetting = s.autoInsertSpaces();
-    s.nospace() << err.asString() << " (code: " << err.code() << ", source: " << err.source() << ")";
-    s.setAutoInsertSpaces(oldSetting);
-    return s.maybeSpace();
-}
-#endif
-
 static QDebug operator<<(QDebug s, const std::vector< std::pair<std::string, std::string> > &v)
 {
     using pair = std::pair<std::string, std::string>;
@@ -326,6 +306,20 @@ static void setDisplaySerialNumber(Card *card, std::shared_ptr<Context> &gpgAgen
     return;
 }
 
+static void learnCardKeyStubs(const Card *card, std::shared_ptr<Context> &gpg_agent)
+{
+    for (const KeyPairInfo &keyInfo : card->keyInfos()) {
+        if (!keyInfo.grip.empty()) {
+            Error err;
+            const auto command = std::string("READKEY --card --no-data -- ") + keyInfo.keyRef;
+            (void)Assuan::sendStatusLinesCommand(gpg_agent, command.c_str(), err);
+            if (err) {
+                qCWarning(KLEOPATRA_LOG) << "Running" << command << "failed:" << err;
+            }
+        }
+    }
+}
+
 static void handle_openpgp_card(std::shared_ptr<Card> &ci, std::shared_ptr<Context> &gpg_agent)
 {
     Error err;
@@ -345,6 +339,8 @@ static void handle_openpgp_card(std::shared_ptr<Card> &ci, std::shared_ptr<Conte
 
     setDisplaySerialNumber(pgpCard, gpg_agent);
 
+    learnCardKeyStubs(pgpCard, gpg_agent);
+
     ci.reset(pgpCard);
 }
 
@@ -410,6 +406,8 @@ static void handle_piv_card(std::shared_ptr<Card> &ci, std::shared_ptr<Context>
         }
     }
 
+    learnCardKeyStubs(pivCard, gpg_agent);
+
     ci.reset(pivCard);
 }
 
@@ -428,12 +426,10 @@ static void handle_p15_card(std::shared_ptr<Card> &ci, std::shared_ptr<Context>
         info.insert(info.end(), fprs.begin(), fprs.end());
     }
 
-    /* Create the key stubs */
-    Assuan::sendStatusLinesCommand(gpg_agent, "READKEY --card --no-data -- $SIGNKEYID", err);
-    Assuan::sendStatusLinesCommand(gpg_agent, "READKEY --card --no-data -- $ENCRKEYID", err);
-
     p15Card->setCardInfo(info);
 
+    learnCardKeyStubs(p15Card, gpg_agent);
+
     setDisplaySerialNumber(p15Card, gpg_agent);
 
     ci.reset(p15Card);
@@ -494,6 +490,8 @@ static void handle_netkey_card(std::shared_ptr<Card> &ci, std::shared_ptr<Contex
         return;
     }
     nkCard->setCardInfo(info);
+
+    learnCardKeyStubs(nkCard, gpg_agent);
 }
 
 static std::shared_ptr<Card> get_card_status(const std::string &serialNumber, const std::string &appName, std::shared_ptr<Context> &gpg_agent)
@@ -875,13 +873,10 @@ public:
         connect(this, &::ReaderStatusThread::anyCardCanLearnKeysChanged,
                 q, &ReaderStatus::anyCardCanLearnKeysChanged);
 
-#ifdef GPGME_SUPPORTS_API_FOR_DEVICEINFOWATCHER
         if (DeviceInfoWatcher::isSupported()) {
             qCDebug(KLEOPATRA_LOG) << "ReaderStatus::Private: Using new DeviceInfoWatcher";
             connect(&devInfoWatcher, &DeviceInfoWatcher::statusChanged, this, &::ReaderStatusThread::deviceStatusChanged);
-        } else
-#endif
-        {
+        } else {
             qCDebug(KLEOPATRA_LOG) << "ReaderStatus::Private: Using deprecated FileSystemWatcher";
 
             watcher.whitelistFiles(QStringList(QStringLiteral("reader_*.status")));
@@ -918,9 +913,7 @@ private:
 
 private:
     FileSystemWatcher watcher;
-#ifdef GPGME_SUPPORTS_API_FOR_DEVICEINFOWATCHER
     DeviceInfoWatcher devInfoWatcher;
-#endif
 };
 
 ReaderStatus::ReaderStatus(QObject *parent)
@@ -940,11 +933,11 @@ ReaderStatus::~ReaderStatus()
 void ReaderStatus::startMonitoring()
 {
     d->start();
-#ifdef GPGME_SUPPORTS_API_FOR_DEVICEINFOWATCHER
     if (DeviceInfoWatcher::isSupported()) {
+        connect(&d->devInfoWatcher, &DeviceInfoWatcher::startOfGpgAgentRequested,
+                this, &ReaderStatus::startOfGpgAgentRequested);
         d->devInfoWatcher.start();
     }
-#endif
 }
 
 // static
