@@ -38,15 +38,11 @@
 #include <QAction>
 #include <QEvent>
 #include <QContextMenuEvent>
+#include <QScrollBar>
 
 #include <KSharedConfig>
 #include <KLocalizedString>
 
-#include <gpgme++/gpgmepp_version.h>
-#if GPGMEPP_VERSION >= 0x10E00 // 1.14.0
-# define GPGME_HAS_REMARKS
-#endif
-
 #define TAGS_COLUMN 13
 
 using namespace Kleo;
@@ -119,12 +115,102 @@ protected:
         return false;
     }
 
+    void keyPressEvent(QKeyEvent *event) override;
+    QModelIndex moveCursor(CursorAction cursorAction, Qt::KeyboardModifiers modifiers) override;
+
 private:
+    bool mMoveCursorUpdatedView = false;
     QMenu *mHeaderPopup = nullptr;
 
     QList<QAction *> mColumnActions;
 };
 
+void TreeView::keyPressEvent(QKeyEvent *event)
+{
+    mMoveCursorUpdatedView = false;
+    QTreeView::keyPressEvent(event);
+    if (mMoveCursorUpdatedView) {
+        event->accept();
+    }
+}
+
+QModelIndex TreeView::moveCursor(CursorAction cursorAction, Qt::KeyboardModifiers modifiers)
+{
+    // this code is based heavily on QTreeView::moveCursor()
+
+    QModelIndex current = currentIndex();
+    if (!current.isValid()) {
+        // let QTreeView handle invalid current index
+        return QTreeView::moveCursor(cursorAction, modifiers);
+    }
+
+    if (isRightToLeft()) {
+        if (cursorAction == MoveRight) {
+            cursorAction = MoveLeft;
+        } else if (cursorAction == MoveLeft) {
+            cursorAction = MoveRight;
+        }
+    }
+    switch (cursorAction) {
+    case MoveLeft: {
+        // HACK: call QTreeView::moveCursor with invalid cursor action to make it call the private executePostedLayout()
+        (void) QTreeView::moveCursor(static_cast<CursorAction>(-1), modifiers);
+
+        int visualColumn = header()->visualIndex(current.column()) - 1;
+        while (visualColumn >= 0 && isColumnHidden(header()->logicalIndex(visualColumn))) {
+            visualColumn--;
+        }
+        int newColumn = header()->logicalIndex(visualColumn);
+        QModelIndex next = current.sibling(current.row(), newColumn);
+        if (next.isValid()) {
+            return next;
+        }
+
+        //last restort: we change the scrollbar value
+        QScrollBar *sb = horizontalScrollBar();
+        int oldValue = sb->value();
+        sb->setValue(sb->value() - sb->singleStep());
+        if (oldValue != sb->value()) {
+            mMoveCursorUpdatedView = true;
+        }
+
+        updateGeometries();
+        viewport()->update();
+        break;
+    }
+    case MoveRight: {
+        // HACK: call QTreeView::moveCursor with invalid cursor action to make it call the private executePostedLayout()
+        (void) QTreeView::moveCursor(static_cast<CursorAction>(-1), modifiers);
+
+        int visualColumn = header()->visualIndex(current.column()) + 1;
+        while (visualColumn < model()->columnCount(current.parent()) && isColumnHidden(header()->logicalIndex(visualColumn))) {
+            visualColumn++;
+        }
+        const int newColumn = header()->logicalIndex(visualColumn);
+        const QModelIndex next = current.sibling(current.row(), newColumn);
+        if (next.isValid()) {
+            return next;
+        }
+
+        //last restort: we change the scrollbar value
+        QScrollBar *sb = horizontalScrollBar();
+        int oldValue = sb->value();
+        sb->setValue(sb->value() + sb->singleStep());
+        if (oldValue != sb->value()) {
+            mMoveCursorUpdatedView = true;
+        }
+
+        updateGeometries();
+        viewport()->update();
+        break;
+    }
+    default:
+        return QTreeView::moveCursor(cursorAction, modifiers);
+    }
+
+    return current;
+}
+
 const KeyListModelInterface * keyListModel(const QTreeView &view)
 {
     const KeyListModelInterface *const klmi = dynamic_cast<KeyListModelInterface *>(view.model());
@@ -255,9 +341,10 @@ void KeyTreeView::init()
 
     m_view->setSelectionBehavior(QAbstractItemView::SelectRows);
     m_view->setSelectionMode(QAbstractItemView::ExtendedSelection);
-    //m_view->setAlternatingRowColors( true );
-    m_view->setAllColumnsShowFocus(true);
+    m_view->setAllColumnsShowFocus(false);
     m_view->setSortingEnabled(true);
+    m_view->setAccessibleName(i18n("Certificates"));
+    m_view->setAccessibleDescription(m_isHierarchical ? i18n("Hierarchical list of certificates") : i18n("List of certificates"));
 
     if (model()) {
         if (m_additionalProxy) {
@@ -292,10 +379,8 @@ void KeyTreeView::init()
                                                      << KeyList::LastUpdate
                                                      << KeyList::Issuer
                                                      << KeyList::SerialNumber
-#ifdef GPGME_HAS_REMARKS
     // If a column is added before this TAGS_COLUMN define has to be updated accordingly
                                                      << KeyList::Remarks
-#endif
     );
     m_view->setModel(rearangingModel);
 
@@ -386,7 +471,6 @@ void KeyTreeView::restoreExpandState()
 
 void KeyTreeView::setUpTagKeys()
 {
-#ifdef GPGME_HAS_REMARKS
     const auto tagKeys = Tags::tagKeys();
     if (m_hierarchicalModel) {
         m_hierarchicalModel->setRemarkKeys(tagKeys);
@@ -394,7 +478,6 @@ void KeyTreeView::setUpTagKeys()
     if (m_flatModel) {
         m_flatModel->setRemarkKeys(tagKeys);
     }
-#endif
 }
 
 void KeyTreeView::saveLayout(KConfigGroup &group)
@@ -598,6 +681,7 @@ void KeyTreeView::setHierarchicalView(bool on)
             m_view->scrollTo(currentIndex);
         }
     }
+    m_view->setAccessibleDescription(m_isHierarchical ? i18n("Hierarchical list of certificates") : i18n("List of certificates"));
     Q_EMIT hierarchicalChanged(on);
 }
 
